@@ -4,6 +4,7 @@ from __future__ import annotations
 from typing import Any, List, Optional, Tuple
 
 from langchain import BasePromptTemplate
+from langchain.prompts import PromptTemplate
 from langchain.schema import (
     AIMessage,
     BaseMessage,
@@ -22,21 +23,24 @@ from kor.type_descriptors import TypeDescriptor
 
 from .validators import Validator
 
+DEFAULT_INSTRUCTION_TEMPLATE = PromptTemplate(
+    input_variables=["type_description", "format_instructions"],
+    template=(
+        "Your goal is to extract structured information from the user's input that"
+        " matches the form described below. When extracting information please make"
+        " sure it matches the type information exactly. Do not add any attributes that"
+        " do not appear in the schema shown below.\n\n"
+        "{type_description}\n\n"
+        "{format_instructions}\n\n"
+    ),
+)
+
 
 class ExtractionPromptValue(PromptValue):
     """Integration with langchain prompt format."""
 
-    text: str
-    encoder: Encoder
-    node: Object
-    type_descriptor: TypeDescriptor
-    input_formatter: InputFormatter
-    prefix: str = (
-        "Your goal is to extract structured information from the user's input that"
-        " matches the form described below. When extracting information please make"
-        " sure it matches the type information exactly. Do not add any attributes that"
-        " do not appear in the schema shown below."
-    )
+    string: str
+    messages: List[BaseMessage]
 
     class Config:
         """Configuration for this pydantic object."""
@@ -44,13 +48,52 @@ class ExtractionPromptValue(PromptValue):
         extra = Extra.forbid
         arbitrary_types_allowed = True
 
-    def get_formatted_text(self) -> str:
-        """Get the text encoded if needed."""
-        return format_text(self.text, input_formatter=self.input_formatter)
-
     def to_string(self) -> str:
+        """Format the prompt to a string."""
+        return self.string
+
+    def to_messages(self) -> List[BaseMessage]:
+        """Get materialized messages."""
+        return self.messages
+
+
+class ExtractionPromptTemplate(BasePromptTemplate):
+    """Extraction prompt template."""
+
+    encoder: Encoder
+    node: Object
+    type_descriptor: TypeDescriptor
+    input_formatter: InputFormatter
+    instruction_template: PromptTemplate
+
+    class Config:
+        """Configuration for this pydantic object."""
+
+        extra = Extra.forbid
+        arbitrary_types_allowed = True
+
+    def format_prompt(  # type: ignore[override]
+        self,
+        text: str,
+    ) -> PromptValue:
+        """Format the prompt."""
+        text = format_text(text, input_formatter=self.input_formatter)
+        return ExtractionPromptValue(
+            string=self.to_string(text), messages=self.to_messages(text)
+        )
+
+    def format(self, **kwargs: Any) -> str:
+        """Implementation of deprecated format method."""
+        raise NotImplementedError()
+
+    @property
+    def _prompt_type(self) -> str:
+        """Prompt type."""
+        return "ExtractionPromptTemplate"
+
+    def to_string(self, text: str) -> str:
         """Format the template to a string."""
-        instruction_segment = self.generate_instruction_segment(self.node)
+        instruction_segment = self.format_instruction_segment(self.node)
         encoded_examples = self.generate_encoded_examples(self.node)
         formatted_examples: List[str] = []
 
@@ -62,14 +105,13 @@ class ExtractionPromptValue(PromptValue):
                 ]
             )
 
-        text = self.get_formatted_text()
         formatted_examples.append(f"Input: {text}\nOutput:")
         input_output_block = "\n".join(formatted_examples)
         return f"{instruction_segment}\n\n{input_output_block}"
 
-    def to_messages(self) -> List[BaseMessage]:
+    def to_messages(self, text: str) -> List[BaseMessage]:
         """Format the template to chat messages."""
-        instruction_segment = self.generate_instruction_segment(self.node)
+        instruction_segment = self.format_instruction_segment(self.node)
 
         messages: List[BaseMessage] = [SystemMessage(content=instruction_segment)]
         encoded_examples = self.generate_encoded_examples(self.node)
@@ -82,8 +124,7 @@ class ExtractionPromptValue(PromptValue):
                 ]
             )
 
-        content = self.get_formatted_text()
-        messages.append(HumanMessage(content=content))
+        messages.append(HumanMessage(content=text))
         return messages
 
     def generate_encoded_examples(self, node: Object) -> List[Tuple[str, str]]:
@@ -93,47 +134,21 @@ class ExtractionPromptValue(PromptValue):
             examples, self.encoder, input_formatter=self.input_formatter
         )
 
-    def generate_instruction_segment(self, node: Object) -> str:
+    def format_instruction_segment(self, node: Object) -> str:
         """Generate the instruction segment of the extraction."""
         type_description = self.type_descriptor.describe(node)
-        instruction_segment = self.encoder.get_instruction_segment()
-        return f"{self.prefix}\n\n{type_description}\n\n{instruction_segment}"
+        format_instructions = self.encoder.get_instruction_segment()
+        input_variables = self.instruction_template.input_variables
 
+        formatting_kwargs = {}
 
-class ExtractionPromptTemplate(BasePromptTemplate):
-    """Extraction prompt template."""
+        if "type_description" in input_variables:
+            formatting_kwargs["type_description"] = type_description
 
-    encoder: Encoder
-    node: Object
-    type_descriptor: TypeDescriptor
-    input_formatter: InputFormatter
+        if "format_instructions" in input_variables:
+            formatting_kwargs["format_instructions"] = format_instructions
 
-    class Config:
-        """Configuration for this pydantic object."""
-
-        extra = Extra.forbid
-        arbitrary_types_allowed = True
-
-    def format_prompt(  # type: ignore[override]
-        self, text: str, **kwargs: Any
-    ) -> PromptValue:
-        """Format the prompt."""
-        return ExtractionPromptValue(
-            text=text,
-            encoder=self.encoder,
-            node=self.node,
-            type_descriptor=self.type_descriptor,
-            input_formatter=self.input_formatter,
-        )
-
-    def format(self, **kwargs: Any) -> str:
-        """Implementation of deprecated format method."""
-        raise NotImplementedError()
-
-    @property
-    def _prompt_type(self) -> str:
-        """Prompt type."""
-        return "ExtractionPromptTemplate"
+        return self.instruction_template.format(**formatting_kwargs)
 
 
 # PUBLIC API
@@ -143,8 +158,10 @@ def create_langchain_prompt(
     schema: Object,
     encoder: Encoder,
     type_descriptor: TypeDescriptor,
+    *,
     validator: Optional[Validator] = None,
     input_formatter: InputFormatter = None,
+    instruction_template: Optional[PromptTemplate] = None,
 ) -> ExtractionPromptTemplate:
     """Create a langchain style prompt with specified encoder."""
     return ExtractionPromptTemplate(
@@ -154,4 +171,5 @@ def create_langchain_prompt(
         node=schema,
         input_formatter=input_formatter,
         type_descriptor=type_descriptor,
+        instruction_template=instruction_template or DEFAULT_INSTRUCTION_TEMPLATE,
     )
